#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    Setup file for wetb.
"""

import os
from git_utils import write_vers
import sys
from setuptools import setup, find_packages
import warnings

repo = os.path.dirname(__file__)
try:
    version = write_vers(vers_file='wetb/__init__.py', repo=repo, skip_chars=1)
except Warning:
    # when there is not git repo, take version string form wetb/__init__.py
    import wetb
    version = wetb.__version__

# try:
#    from pypandoc import convert_file
#    read_md = lambda f: convert_file(f, 'rst', format='md')
# except ImportError:
#    print("warning: pypandoc module not found, could not convert Markdown to RST")
#    read_md = lambda f: open(f, 'r').read()
with open("README.md", "r") as fh:
    long_description = fh.read()


from distutils.extension import Extension


def setup_package(build_ext_switch=True):
    if build_ext_switch:
        import numpy as np
        ex_info = [('wetb.fatigue_tools.rainflowcounting', ['pair_range', 'peak_trough', 'rainflowcount_astm']),
                   ('wetb.signal.filters', ['cy_filters'])]
        extlist = [Extension('%s.%s' % (module, n),
                             [os.path.join(module.replace(".", "/"), n) + '.pyx'],
                             include_dirs=[np.get_include()]) for module, names in ex_info for n in names]
        from Cython.Distutils import build_ext
        build_requires = ['cython']
        cmd_class = {'build_ext': build_ext}
    else:
        extlist = []
        build_requires = []
        cmd_class = {}

    needs_sphinx = {'build_sphinx', 'upload_docs'}.intersection(sys.argv)
    sphinx = ['sphinx'] if needs_sphinx else []
    install_requires = ['future',
                        'h5py',
                        'tables',
                        'pytest',
                        'pytest-cov',
                        #                        'blosc', # gives an error - has to be pre-installed
                        'pbr',
                        'paramiko',
                        'scipy',
                        'pandas',
                        'matplotlib',
                        'cython',
                        'xlrd',
                        'coverage',
                        'xlwt',
                        'openpyxl',
                        'psutil',
                        'six',
                        'sshtunnel',
                        'click',
                        'jinja2', ]

    setup(install_requires=install_requires,
          setup_requires=install_requires + build_requires + sphinx,
          cmdclass=cmd_class,
          ext_modules=extlist,
          long_description=long_description,
          long_description_content_type="text/markdown",
          version=version,
          packages=find_packages(),
          )


if __name__ == "__main__":
    try:
        setup_package()
    except Exception:
        setup_package(build_ext_switch=False)
        warnings.warn("WETB installed, but building extensions failed (i.e. it falls back on the slower pure python implementions)",
                      RuntimeWarning)



'''
Created on 28. jul. 2017

@author: mmpe
'''
import os
import subprocess


def _run_git_cmd(cmd, git_repo_path=None):
    git_repo_path = git_repo_path or os.getcwd()
    if not os.path.isdir(os.path.join(git_repo_path, ".git")):
        raise Warning("'%s' does not appear to be a Git repository." % git_repo_path)
    try:
        process = subprocess.Popen(cmd,
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.PIPE,
                                   universal_newlines=True,
                                   cwd=os.path.abspath(git_repo_path))
        stdout,stderr = process.communicate()
        if process.returncode != 0:
            raise EnvironmentError("%s\n%s"%(stdout, stderr))
        return stdout.strip()

    except EnvironmentError as e:
        raise e
        raise Warning("unable to run git")


def get_git_branch(git_repo_path=None):
    cmd = ["git", "rev-parse", "--abbrev-ref", "HEAD"]
    return _run_git_cmd(cmd, git_repo_path)


def get_git_version(git_repo_path=None):
    cmd = ["git", "describe", "--tags", "--dirty", "--always"]
    return _run_git_cmd(cmd, git_repo_path)


def get_tag(git_repo_path=None, verbose=False):
    tag = _run_git_cmd(['git', 'describe', '--tags', '--always', '--abbrev=0'],
                       git_repo_path)
    if verbose:
        print(tag)
    return tag


def set_tag(tag, push, git_repo_path=None):
    _run_git_cmd(["git", "tag", tag], git_repo_path)
    if push:
        _run_git_cmd(["git", "push"], git_repo_path)
        _run_git_cmd(["git", "push", "--tags"], git_repo_path)


def update_git_version(version_module, git_repo_path=None):
    """Update <version_module>.__version__ to git version"""

    version_str = get_git_version(git_repo_path)
    assert os.path.isfile(version_module.__file__)
    with open(version_module.__file__, "w") as fid:
        fid.write("__version__ = '%s'" % version_str)

    # ensure file is written, closed and ready
    with open(version_module.__file__) as fid:
        fid.read()
    return version_str


def write_vers(vers_file='wetb/__init__.py', repo=None, skip_chars=1):
    """Writes out version string as follows:
        "last tag"-("nr commits since tag")-("branch name")-("hash commit")
    and where nr of commits since last tag is only included if >0,
    branch name is only inlcuded when not on master,
    and hash commit is only included when not at a tag (when nr of commits > 0)
    """
    if not repo:
        repo = os.getcwd()
    version_long = get_git_version(repo)
    branch = get_git_branch(repo)

    verel = version_long.split('-')
    # tag name
    version = verel[0][skip_chars:]
    # number of commits since last tag, only if >0
    nr_commits = 0
    if len(verel) > 1:
        try:
            nr_commits = int(verel[1])
        except ValueError:
            nr_commits = -1
        if nr_commits > 0:
            version += '-' + verel[1]
    # branch name, only when NOT on master
    if branch != 'master':
        version += '-' + branch
    # hash commit, only if not at tag
    if len(verel) > 2 and nr_commits > 0:
        # first character on the hash is always a g (not part of the hash)
        version += '-' + verel[2][1:]
    # if "-HEAD" is added to the version, which pypi does not like:
    if version.endswith('-HEAD'):
        version = version[:-5]
    print(version_long)
    print('Writing version: {} in {}'.format(version, vers_file))

    with open(vers_file, 'r') as f:
        lines = f.readlines()
    for n, l in enumerate(lines):
        if l.startswith('__version__'):
            lines[n] = "__version__ = '{}'\n".format(version)
    for n, l in enumerate(lines):
        if l.startswith('__release__'):
            lines[n] = "__release__ = '{}'\n".format(version)
    with open(vers_file, 'w') as f:
        f.write(''.join(lines))
    return version


def rename_dist_file():
    for f in os.listdir('dist'):
        if f.endswith('whl'):
            split = f.split('linux')
            new_name = 'manylinux1'.join(split)
            old_path = os.path.join('dist', f)
            new_path = os.path.join('dist', new_name)
            os.rename(old_path, new_path)


def main():
    """Example of how to run (pytest-friendly)"""
    if __name__ == '__main__':
        pass


main()



import itertools
import jinja2
import pandas as pd
import click
import os
from pathlib import Path
from pandas.core.base import PandasObject
from wetb.hawc2.htc_file import HTCFile


class HAWC2InputWriter(object):
    """
    Base HAWC2InputWriter object class, using the tagless system. Subclasses are:
     - JinjaWriter
    """

    def __init__(self, base_htc_file, **kwargs):
        self.base_htc_file = base_htc_file
        self.contents = None
        for k, v in kwargs.items():
            setattr(self, k, v)

    def __call__(self, out_fn, **kwargs):
        return self.write(out_fn, **kwargs)

    def from_pandas(self, dataFrame):
        """
        Loads a DataFrame of contents from a PandasObject

        Args:
            dataFrame (Pandas.DataFrame of PandasObject): Dataframe of contents

        Returns:
            self (HAWC2InputWriter): Instance of self to enable chaining.
        """

        if not isinstance(dataFrame, PandasObject) and hasattr(dataFrame, 'to_pandas'):
            dataFrame = dataFrame.to_pandas()

        if isinstance(dataFrame, dict):
            return {n: self.from_pandas(df.dropna(how='all')) for n, df in dataFrame.iteritems()}

        self.contents = dataFrame

        return self

    def from_excel(self, excel_file):
        """
        Loads a DataFrame of contents from an excel file

        Args:
            excel_file (str, Pathlib.Path): path to excel file.

        Returns:
            self (HAWC2InputWriter): Instance of self to enable chaining.
        """

        self.contents = pd.read_excel(excel_file, keep_default_na=False)
        if 'Unnamed: 0' in self.contents:
            self.contents = self.contents.drop(columns=['Unnamed: 0'])
        return self

    def from_CVF(self, constants, variables, functionals):
        """
        Produces a DataFrame of contents given dictionaries of constants, variables
        and functionals.

        Args:
            constants (dict): content, value pairs for which the value remains
                constant for all generated htc files.
            variables (dict of list): content, list pairs for which all combinations
                of the listed values are generated in the htc files.
            functionals (dict of functions): content, function pairs for which the
                content is dependent on the constants and variables contents.

        Returns: self (HAWC2InputWriter): Instance of self to enable chaining.
        """

        attributes = list(constants.keys()) + list(variables.keys()) + list(functionals.keys())
        contents = []

        for var in itertools.product(*list(variables.values())):
            this_dict = dict(constants)
            var_dict = dict(zip(variables.keys(), var))
            this_dict.update(var_dict)

            for key, func in functionals.items():
                this_dict[key] = func(this_dict)

            contents.append(list(this_dict.values()))

        self.contents = pd.DataFrame(contents, columns=attributes)

        return self

    def write(self, out_fn, **kwargs):
        '''
        Renders a single htc file for a given set of 'tagless' contents.
        args:
        out_fn (str or pathlib.Path): The output filename where to render
        the jinja template.
        params (pd.DataFrame or pd.Series) : The input contents.
        '''
        # if isinstance(params, PandasObject):
        #     params = params.to_dict()

        htc = HTCFile(self.base_htc_file, jinja_tags=kwargs)
        for k, v in kwargs.items():
            k = k.replace('/', '.')
            if '.' in k:
                line = htc[k]
                v = str(v).strip().replace(",", " ")
                line.values = v.split()
            else:
                if hasattr(self, 'set_%s' % k):
                    getattr(self, 'set_%s' % k)(htc, **kwargs)

            htc.save(out_fn)

    def write_all(self, out_dir):
        '''
        Renders all htc files for the set of contents.
        args:
            out_dir (str or pathlib.Path): The directory where the htc files are generated.
        '''
        out_dir = Path(out_dir)
        out_dir.mkdir(parents=True, exist_ok=True)

        N = len(self.contents)

        print(f'Generating {N} htc files in directory: {out_dir}')

        with click.progressbar(self.contents.iterrows(), length=N) as bar:
            for _, row in bar:
                if 'Folder' in row and row.Folder:
                    path = out_dir / row.Folder
                else:
                    path = out_dir

                self.write(path / (row.Name + '.htc'), **row.to_dict())


class JinjaWriter(HAWC2InputWriter):
    """
    !!! DEPRECATED
    !!! HAWC2InputWriter accepts jinja tags
    !!!
    Subclass of the HAWC2InputWriter object. Generates htc files using contents compatible with jinja2.
    """

    def write(self, out_fn, **kwargs):
        params = pd.Series(kwargs)
        with open(self.base_htc_file) as f:
            template = jinja2.Template(f.read())

            out = template.render(params)

            with open(out_fn, 'w') as f:
                f.write(out)


def main():
    if __name__ == '__main__':
        from wetb.hawc2.tests import test_files

        # HAWC2InputWriter example
        path = os.path.dirname(test_files.__file__) + '/simulation_setup/DTU10MWRef6.0/'
        base_htc_file = path + 'htc/DTU_10MW_RWT.htc'

        class MyWriter(HAWC2InputWriter):
            def set_time(self, htc, time, **_):
                htc.set_time(self.time_start, self.time_start + time)

        myWriter = MyWriter(base_htc_file, time_start=100)
        myWriter('tmp/t1.htc', Name="t1", time=600, **{"wind.wsp": 10})

        # JinjaWriter example with constants, variables, and functionals
        Constants = {
            'time_stop': 100,
        }

        Variables = {
            'wsp': [4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24],
            'tint': [0.05, 0.10, 0.15],

        }

        Functionals = {
            'Name': lambda x: 'dtu10mw_wsp' + str(x['wsp']) + '_ti' + str(x['tint']),
        }

        htc_template_fn = os.path.dirname(test_files.__file__) + \
            '/simulation_setup/DTU10MWRef6.0/htc/DTU_10MW_RWT.htc.j2'
        writer = JinjaWriter(htc_template_fn)
        writer.from_CVF(Constants, Variables, Functionals)
        print(writer.contents)

        # write all htc files to folder 'htc'
        writer.write_all('htc')


main()
